/*!
  \file   parms_vec_impl.h
  \brief  parms_Vec structure.

  \author zzli
  \date   2006-05-05
*/

#ifndef _PARMS_VEC_IMPL_H_
#define _PARMS_VEC_IMPL_H_

#if defined(__ICC)
#include <mathimf.h>
#else
#if defined(C99)
#include <tgmath.h>
#else
#include <math.h>
#endif 
#endif 

#include "parms_viewer.h"
#include "parms_map_impl.h"
#include "parms_vec.h"

/*! \typedef parms_Vec_ops
  \brief array of function pointers associated with the parms_Vec object.
 */
typedef struct parms_Vec_ops {
  int    (*getlsize)(parms_Vec self);
  int    (*getgsize)(parms_Vec self);
  int    (*getarray)(parms_Vec self, FLOAT **array);
  int    (*restore)(parms_Vec self, FLOAT **array);
  int    (*norm2)(parms_Vec self, double *value);
  int    (*put)(parms_Vec self, FLOAT value);
  int    (*scale)(parms_Vec self, FLOAT scalar);
  int    (*setvalues)(parms_Vec self, int m, int *im, FLOAT *values,
		      INSERTMODE mode);  
  int    (*axpy)(parms_Vec self, parms_Vec x, FLOAT scalar);
  int    (*aypx)(parms_Vec self, parms_Vec x, FLOAT scalar);
  int    (*dot)(parms_Vec self, parms_Vec x, FLOAT *value);
  int    (*dotarray)(parms_Vec self, int n, parms_Vec *parms_Vecarray,
		     FLOAT *aux, FLOAT *result);
  int    (*attvec)(parms_Vec self, int start, parms_Vec x);
  int    (*attarray)(parms_Vec self, FLOAT *array);
  int    (*setup)(parms_Vec self);
} *parms_Vec_ops;

/*! \struct parms_Vec_
  \brief parms_Vec_ structure.
 */
struct  parms_Vec_ {
  PARMS_OBJHEADER(parms_Vec_ops)
  BOOL      issetup;		//!< setup or not
  BOOL      isperm;		//!< permuted or not
  BOOL      isatt;		//!< attached or not
  BOOL      isalloc;		//!< allocated or not
  int       lsize;		//!< local size of the parms_Vec
  int       gsize;		//!< global size of the parms_Vec
  parms_Map is;			//!< data layout object
  FLOAT     *array;		//!< local array
};

#endif  
