#ifndef __ITSOL_INCLUDED_PROTOS_H__
#define __ITSOL_INCLUDED_PROTOS_H__

#include "globheads.h"

#if defined(FORTRAN_CAPS)
#define qsplit  QSPLIT
#define readmtc READMTC
#define csrcsc  CSRCSC
#define roscal  ROSCAL
#define coscal  COSCAL
#elif defined(FORTRAN_DOUBLE_UNDERSCORE)
#define qsplit  qsplit__
#define readmtc readmtc__
#define csrcsc  csrcsc__
#define roscal  roscal__
#define coscal  coscal__
#elif defined(FORTRAN_UNDERSCORE)
#define qsplit  qsplit_
#define readmtc readmtc_
#define csrcsc  csrcsc_
#define roscal  roscal_
#define coscal  coscal_
#else
#include <essl.h>
#endif

#ifndef min
#define min(a,b) (((a)>(b))?(b):(a))
#endif
#ifndef max
#define max(a,b) (((a)>(b))?(a):(b))
#endif

#define MAX_LINE        256
#define MAX_HBNAME      64

/* FORTRAN routines */
extern void readmtc(int*,  int*,  int*,  char*,  double*,  int*,
	     int*,  double*, int*,  char*,  int*,  int*,  int*,
	     char*,  char*, char*,  int*) ;
extern void csrcsc (int*, int*, int*, double*, int*, int*, double*,
		    int*, int*) ; 
extern void qsplit(double *a, int *ind, int *n, int *ncut);	

/* sets */
extern void errexit(char *f_str, ...);
extern void *Malloc(int nbytes, char *msg); 
extern int setupP4 (p4ptr amat, int Bn, int Cn,  csptr F,  csptr E);
extern int setupILUT(ilutptr amat, int len);
extern int setupCS(csptr amat, int len, int job); 
extern int cleanCS(csptr amat);
extern int nnzCS(csptr amat); 
extern int nnz_arms (arms PreSt);
extern int cs_nnz (csptr A) ;
extern int cscpy(csptr amat, csptr bmat);
extern int setupVBMat(vbsptr vbmat, int n, int *nB);
extern int cleanVBMat(vbsptr vbmat); 
extern int nnzVBMat(vbsptr vbmat) ;
extern int memVBMat(vbsptr vbmat); 
extern int setupVBILU(vbiluptr lu, int n, int *bsz);
extern int cleanVBILU(vbiluptr lu); 
extern int cleanILU( iluptr lu );
extern int mallocVBRow(vbiluptr lu, int nrow); 
extern int mallocRow( iluptr lu, int nrow );
extern void zrmC(int m, int n, BData data); 
extern void copyBData(int m, int n, BData dst, BData src, int isig);
extern int CSRcs(int n, double *a, int *ja, int *ia, csptr mat, int rsa);
extern int csrvbsrC(int job, int nBlk, int *nB, csptr csmat, vbsptr vbmat); 
extern int col2vbcol( int col, vbsptr vbmat );
extern int nnz_vbilu(vbiluptr lu); 
extern int setupILU( iluptr lu, int n );

/* MatOps */
extern int diag_scal(vbsptr vbmat);
extern int diagvec(vbsptr vbmat, BData x, BData y);
extern void matvec(csptr mata, double *x, double *y); 
extern void matvecz(csptr mata, double *x, double *y, double *z);
extern void vbmatvec(vbsptr vbmat, double *x, double *y);
extern void luinv(int n, double *a, double *x, double *y); 
extern int vblusolC(double *y, double *x, vbiluptr lu); 
extern int lusolC( double *y, double *x, iluptr lu ); 
extern int rpermC(csptr mat, int *perm); 
extern int cpermC(csptr mat, int *perm) ; 
extern int dpermC(csptr mat, int *perm) ; 
extern int CSparTran(csptr amat, csptr bmat, CompressType *compress);
extern double vbnorm2(int sz, double *a);
extern void invsp(int start, ilutptr ilusch, double *y, double *x);
extern int armsol2(double *x,  arms Prec);
extern int ascend (p4ptr levmat, double *x, double *wk);
extern int descend(p4ptr levmat, double *x, double *wk);
extern p4ptr Lvsol2(double *x, int nlev, p4ptr levmat, ilutptr ilusch,
		    int flag); 
extern int   Uvsol2(double *x, int nlev, int n, p4ptr levmat, ilutptr
		    ilusch); 
extern void  SchLsol(ilutptr ilusch, double *y) ;
extern void  SchUsol(ilutptr ilusch, double *y) ;
extern void Lsolp(int start, csptr mata, double *b, double *y);
extern void Usolp(int start, csptr mata, double *y, double *x);
extern int invGauss(int nn, double *A); 
extern int invSVD(int nn, double *A) ;

/* setblks.c */ 
extern int KeyComp(const void *vfst, const void *vsnd);
extern int init_blocks(csptr, int *, int **, int **, double, double *,
		       double *);  

/* upper directory */
extern int vbilukC( int lofM, vbsptr vbmat, vbiluptr lu, FILE *fp ); 
extern int vbilutC( vbsptr vbmat, vbiluptr lu, int lfil, double tol,
		    BData *w, FILE *fp ); 
extern int ilutc(iluptr mt, iluptr lu, int lfil, double tol, int drop,
		 FILE *fp ); 
extern int VBcondestC( vbiluptr lu, double *y, double *x, FILE *fp );
extern int ilukC( int lofM, csptr csmat, iluptr lu, FILE *fp );
extern int ilut( csptr csmat, iluptr lu, int lfil, double tol,
		 double *w, FILE *fp );
extern int condestLU( iluptr lu, double *y, double *x, FILE *fp );
extern int nnz_ilu( iluptr lu ); 
extern void roscal (int* nrow, int* job, int* nrm, double *a, int *ja,
		    int *ia, double *diag, double *b, int *jb, int
		    *ib, int *ierr) ;  
extern void coscal (int* nrow, int* job, int* nrm, double *a, int *ja,
		    int *ia, double *diag, double *b, int *jb, int
		    *ib, int *ierr) ;  
extern int outputLU( iluptr lu, char *filename );
extern int lumsolC(double *y, double *x, iluptr lu );
extern void lumatvec(iluptr mat, double *x, double *y);
extern int CSClum( int n, double *a, int *ja, int *ia, iluptr mat, int rsa ); 
extern void setup_arms (arms Levmat);
extern int cleanARMS(arms ArmsPre);
extern int csSplit4(csptr amat, int bsize, int csize, csptr B, csptr F,
		    csptr E, csptr C); 
/* misc.c */
extern int SparTran(csptr amat, csptr bmat, int job, int flag); 
extern int coscalC(csptr mata, double *diag, int nrm);
extern void dscale(int n, double *dd, double *x, double * y);
extern void hilosort(csptr mat, int abval, int hilo);
extern void printmat(FILE *ft, csptr A, int i0, int i1);
extern void qqsort(int *ja, double *ma, int left, int right);
extern void qsort2C(int *ja, double *ma, int left, int right, int
		    abval); 
extern void qsort3i(int *wa, int *cor1, int *cor2, int left, int
		    right); 
extern void qsortC(int *ja, double *ma, int left, int right, int
		   abval); 
extern void qsortR2I(double *wa, int *cor1, int *cor2, int left, int
		     right); 
extern int qsplitC(double *a, int *ind, int n, int ncut);
extern int roscalC(csptr mata, double *diag, int nrm);
extern void swapj(int v[], int i, int j);
extern void swapm(double v[], int i, int j);

/* piluNEW.c */
extern int pilu(p4ptr amat, csptr B, csptr C, double *droptol, int
		*lfil, csptr schur);

/* ilutpC.c */
extern int ilutD(csptr amat, double *droptol, int *lfil, ilutptr
		 ilusch);
extern int ilutpC(csptr amat, double *droptol, int *lfil, double
		  permtol, int mband, ilutptr ilusch);

/* PQ.c */
extern int PQperm(csptr mat, int bsize, int *Pord, int *Qord, int
		  *nnod, double tol, int nbnd);
extern int preSel(csptr mat, int *icor, int *jcor, int job, double
		  tol, int *count, int nbnd);
extern int weightsC(csptr mat, double *w);
extern int add2com(int *nback, int nod, int *iord, int *riord);
extern int add2is(int *last, int nod, int *iord, int *riord);
extern int indsetC(csptr mat, int bsize, int *iord, int *nnod, double
		   tol,int nbnd); 

/* setblks.c */
extern int KeyComp( const void *vfst, const void *vsnd );
extern int init_blocks( csptr csmat, int *pnBlock, int **pnB, int
			**pperm, double eps, double *t_hash, double
			*t_angle );

/* systimer.c */
extern double sys_timer();

#endif 
